/*******************************************
 *File: engine.h
 *Author: Peter Asplund
 *Created: 2004-11-08
 *Description: Simulates the engine. Receives boolean wether to accelerate or not. If true, it calculates the RPM and the power. It also handles the gears and brakes.
 ******************************************/

#ifndef ENGINE_H
#define ENGINE_H

#include <vector>

using namespace std;

namespace engine {

  class Engine {
    
  public:
  
    Engine();
    bool setGearUp();
    bool setGearDown();
    void setMaxSpeed(float);
    void calculateAcceleration(bool&, bool&, float&);
    double getSpeed() const;
    double getForce() const;
    int getGear() const;
    double getRPM() const;
    
  private:
  
    double stepSize;
    bool isAccelerating;
    bool isBreaking;
    double dX1;
    double dU1;

    /****************************************
     * Variables that are calculated in Euler
     ***************************************/
    double speed;             //speed after acceleration = X2(t+h)
    double uSpeed;

    double drivingForce;      //Force to drive the tank
    double angleVel;          //Angle velocity in engine
    double torque;            //force from gasoline (constant for now)
    double brakeFriction;     //Friciton in brakes

    /******************
     * Vehicle Handling
     *****************/
    int currentGear;
    float maxSpeed;

    /********************
     * Constant variables
     *******************/
    static const double INTERNALFRICTION = 105;
    static const double INTERNALMOMENTUM = 100;
    static const double CHASSIMASS = 60000;
    static const double GROUNDFRICTION = 40;
    float TRACKELASTICITY;
    vector<float> transmission; //transmission.at(currentGear) = r

  };

} //end of namespace engine

#endif //ENGINE_H

